/*****************************************************************************
 *   irq.c: Interrupt handler C file for Philips LPC214x Family Microprocessors
 *
 *   Copyright(C) 2006, Philips Semiconductor
 *   All rights reserved.
 *
 *   History
 *   2005.10.01  ver 1.00    Prelimnary version, first Release
 *
******************************************************************************/
#include "lpc214x.h"			/* LPC21XX Peripheral Registers	*/
#include "irq.h"
#include "defines.h"

#define IRQ_MASK 0x00000080
#define FIQ_MASK 0x00000040
#define INT_MASK (IRQ_MASK | FIQ_MASK)

/******************************************************************************
** Function name:		DefaultVICHandler
**
** Descriptions:		Default VIC interrupt handler.
**				This handler is set to deal with spurious 
**				interrupt.
**				If the IRQ service routine reads the VIC
**				address register, and no IRQ slot responses
**				as described above, this address is returned.
** parameters:			None
** Returned value:		None
** 
******************************************************************************/
// mthomas: inserted static to avoid gcc-warning
static void __irq DefaultVICHandler (void)
{
	ISR_ENTER();
    /* if the IRQ is not installed into the VIC, and interrupt occurs, the
    default interrupt VIC address will be used. This could happen in a race 
    condition. For debugging, use this endless loop to trace back. */
    /* For more details, see Philips appnote AN10414 */
    
    ISR_EXIT(); 
}

/* Initialize the interrupt controller */
/******************************************************************************
** Function name:		initVIC
**
** Descriptions:		Initialize VIC interrupt controller.
** parameters:			None
** Returned value:		None
** 
******************************************************************************/
void irq_init(void) 
{
    uint32_t i = 0;
    uint32_t *vect_addr, *vect_cntl;
   	
    /* initialize VIC*/
    VICIntEnClr  = 0xffffffff;
    VICVectAddr  = 0;
    VICIntSelect = 0;

    /* set all the vector and vector control register to 0 */
    for ( i = 0; i < VIC_SIZE; i++ )
    {
		vect_addr  = (uint32_t *)(VIC_BASE_ADDR + VECT_ADDR_INDEX + i*4);
		vect_cntl  = (uint32_t *)(VIC_BASE_ADDR + VECT_CNTL_INDEX + i*4);
		*vect_addr = 0;	
		*vect_cntl = 0;
    }

    /* Install the default VIC handler here */
    VICDefVectAddr = (uint32_t)DefaultVICHandler;   
	
    return;
}

/******************************************************************************
** Function name:		irq_install
**
** Descriptions:		Install interrupt handler
**				The max VIC size is 16, but, there are 32 interrupt
**				request inputs. Not all of them can be installed into
**				VIC table at the same time.
**				The order of the interrupt request installation is
**				first come first serve.
** parameters:			Interrupt number and interrupt handler address
** Returned value:		true or false, when the table is full, return false
** 
******************************************************************************/
uint32_t irq_install( uint32_t IntNumber, void *HandlerAddr )
{
    uint32_t i;
    uint32_t *vect_addr;
    uint32_t *vect_cntl;
      
    VICIntEnClr = 1 << IntNumber;	/* Disable Interrupt */
    
    for ( i = 0; i < VIC_SIZE; i++ )
    {
		/* find first un-assigned VIC address for the handler */
		
		vect_addr = (uint32_t *)(VIC_BASE_ADDR + VECT_ADDR_INDEX + i*4);
		vect_cntl = (uint32_t *)(VIC_BASE_ADDR + VECT_CNTL_INDEX + i*4);
		if ( *vect_addr == (uint32_t)NULL )
		{
			*vect_addr = (uint32_t)HandlerAddr;	/* set interrupt vector */
			*vect_cntl = (uint32_t)(IRQ_SLOT_EN | IntNumber);
			break;
		}
    }
	
    if ( i == VIC_SIZE )
    {
		return( FALSE );		/* fatal error, can't find empty vector slot */
    }
	
    VICIntEnable = 1 << IntNumber;	/* Enable Interrupt */
	
    return( TRUE );
}

/******************************************************************************
** Function name:		uninstall_irq
**
** Descriptions:		Uninstall interrupt handler
**				Find the interrupt handler installed in the VIC
**				based on the interrupt number, set the location
**				back to NULL to uninstall it.
** parameters:			Interrupt number
** Returned value:		true or false, when the interrupt number is not found, 
**				return false
** 
******************************************************************************/
uint32_t irq_uninstall( uint32_t IntNumber )
{
    uint32_t i;
    uint32_t *vect_addr;
    uint32_t *vect_cntl;
      
    VICIntEnClr = 1 << IntNumber;	/* Disable Interrupt */
    
    for ( i = 0; i < VIC_SIZE; i++ )
    {
		/* find first assigned VIC address for the handler */
		vect_addr = (uint32_t *)(VIC_BASE_ADDR + VECT_ADDR_INDEX + i*4);
		vect_cntl = (uint32_t *)(VIC_BASE_ADDR + VECT_CNTL_INDEX + i*4);
		if ( (*vect_cntl & ~IRQ_SLOT_EN ) == IntNumber )
		{
			*vect_addr  = (uint32_t)NULL;  /* clear the VIC entry in the VIC table */
			*vect_cntl &= ~IRQ_SLOT_EN;	  /* disable SLOT_EN bit */	
			break;
		}
    }
	
    if ( i == VIC_SIZE )
    {
		return( FALSE );		/* fatal error, can't find interrupt number in vector slot */
    }
	
    VICIntEnable = 1 << IntNumber;	/* Enable Interrupt */
	
    return( TRUE );
}

static inline uint32_t __get_cpsr(void)
{
  unsigned long retval;
  asm volatile (" mrs  %0, cpsr" : "=r" (retval) : /* no inputs */  );
  return retval;
}

static inline void __set_cpsr(uint32_t val)
{
  asm volatile (" msr  cpsr, %0" : /* no outputs */ : "r" (val)  );
}

/* Enable and disable functions "stripped" from a sample by R O Software.
 * Copyright 2004, R O SoftWare
 * No guarantees, warrantees, or promises, implied or otherwise.
 * May be used for hobby or commercial purposes provided copyright
 * notice remains intact. */
 

#define IRQ_MASK 0x00000080

static inline uint32_t asm_get_cpsr(void)
{
  unsigned long retval;
  asm volatile (" mrs  %0, cpsr" : "=r" (retval) : /* no inputs */  );
  return retval;
}

static inline void asm_set_cpsr(uint32_t val)
{
  asm volatile (" msr  cpsr, %0" : /* no outputs */ : "r" (val)  );
}

uint32_t irq_enableIRQ(void)
{
  uint32_t _cpsr;

  _cpsr = asm_get_cpsr();
  asm_set_cpsr(_cpsr & ~IRQ_MASK);
  return _cpsr;
}

uint32_t irq_disableIRQ(void)
{
  uint32_t _cpsr;

  _cpsr = asm_get_cpsr();
  asm_set_cpsr(_cpsr | IRQ_MASK);
  return _cpsr;
}

uint32_t irq_restoreIRQ(uint32_t oldCPSR)
{
  uint32_t _cpsr;

  _cpsr = asm_get_cpsr();
  asm_set_cpsr((_cpsr & ~IRQ_MASK) | (oldCPSR & IRQ_MASK));
  return _cpsr;
}
/* end of R O code */
